/*
 * $Header: /home/cvs/jakarta-commons/jxpath/src/java/org/apache/commons/jxpath/ri/pointers/PropertyAccessHelper.java,v 1.1 2001/08/23 00:47:00 dmitri Exp $
 * $Revision: 1.1 $
 * $Date: 2001/08/23 00:47:00 $
 *
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, Plotnix, Inc,
 * <http://www.plotnix.com/>.
 * For more information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.commons.jxpath.ri.pointers;

import org.apache.commons.jxpath.*;
import org.apache.commons.jxpath.ri.Compiler;
import org.apache.commons.jxpath.ri.compiler.*;

import java.lang.reflect.*;
import java.util.*;
import java.beans.*;

/**
 * @author Dmitri Plotnikov
 * @version $Revision: 1.1 $ $Date: 2001/08/23 00:47:00 $
 */
public class PropertyAccessHelper {
    private static Map dynamicPropertyHandlerMap = new HashMap();

    public static boolean isCollection(PropertyDescriptor propertyDescriptor){
        return false;
    }

    public static boolean isCollection(PropertyDescriptor propertyDescriptor, Object value){
        return isCollection(value);
    }

    public static boolean isCollection(Object value){
        if (value == null){
            return false;
        }
        else if (value.getClass().isArray()){
            return true;
        }
        else if (value instanceof Collection){
            return true;
        }
        return false;
    }

    public static int getLength(Object bean, PropertyDescriptor propertyDescriptor){
        Object obj = getValue(bean, propertyDescriptor);
        return getLength(obj);
    }

    public static int getLength(Object collection){
        if (collection == null){
            return 0;
        }
        else if (collection.getClass().isArray()){
            return Array.getLength(collection);
        }
        else if (collection instanceof Collection){
            return ((Collection)collection).size();
        }
        else {
            return 1;
        }
    }

    public static Object getValue(Object bean, PropertyDescriptor propertyDescriptor, int index){
        if (propertyDescriptor instanceof IndexedPropertyDescriptor){
            Object value;
            try {
                IndexedPropertyDescriptor ipd = (IndexedPropertyDescriptor)propertyDescriptor;
                Method method = ipd.getIndexedReadMethod();
                if (method != null){
                    return method.invoke(bean, new Object[]{new Integer(index)});
                }
            }
            catch (Exception ex){
                throw new RuntimeException("Cannot access property: " + propertyDescriptor.getName() +
                    ", " + ex.getMessage());
            }
        }
        // We will fall through if there is no indexed read

        return getValue(getValue(bean, propertyDescriptor), index);
    }

    public static void setValue(Object bean, PropertyDescriptor propertyDescriptor, int index, Object value){
        if (propertyDescriptor instanceof IndexedPropertyDescriptor){
            try {
                IndexedPropertyDescriptor ipd = (IndexedPropertyDescriptor)propertyDescriptor;
                Method method = ipd.getIndexedWriteMethod();
                if (method != null){
                    method.invoke(bean, new Object[]{new Integer(index), value});
                    return;
                }
            }
            catch (Exception ex){
                throw new RuntimeException("Cannot access property: " + propertyDescriptor.getName() +
                    ", " + ex.getMessage());
            }
        }
        // We will fall through if there is no indexed read
        setValue(getValue(bean, propertyDescriptor), index, value);
    }

    public static Object getValue(Object collection, int index){
        Object value = collection;
        if (collection != null){
            if (collection.getClass().isArray()){
                value = Array.get(collection, index);
            }
            else if (collection instanceof List){
                value = ((List)collection).get(index);
            }
            else if (collection instanceof Collection){
                Iterator it = ((Collection)collection).iterator();
                for (int i = 0; i <= index; i++){
                    value = it.next();
                }
            }
        }
        return value;
    }

    public static void setValue(Object collection, int index, Object value){
        if (collection != null){
            if (collection.getClass().isArray()){
                Array.set(collection, index, value);
            }
            else if (collection instanceof List){
                ((List)collection).set(index, value);
            }
            else if (collection instanceof Collection){
                throw new UnsupportedOperationException("Cannot set value of an element of a " +
                        collection.getClass().getName());
            }
        }
    }

    public static Object getValue(Object bean, PropertyDescriptor propertyDescriptor){
        Object value;
        try {
            Method method = propertyDescriptor.getReadMethod();
            if (method == null){
                throw new RuntimeException("No read method");
            }
            value = method.invoke(bean, new Object[0]);
        }
        catch (Exception ex){
            throw new RuntimeException("Cannot access property: " + propertyDescriptor.getName() +
                ", " + ex.getMessage());
        }
        return value;
    }

    public static void setValue(Object bean, PropertyDescriptor propertyDescriptor, Object value){
        try {
            Method method = propertyDescriptor.getWriteMethod();
            if (method == null){
                throw new RuntimeException("No write method");
            }
            value = method.invoke(bean, new Object[]{value});
        }
        catch (Exception ex){
            throw new RuntimeException("Cannot modify property: " + propertyDescriptor.getName() +
                ", " + ex);
        }
    }

    /**
     * Returns a shared instance of the dynamic property handler class
     * returned by <code>getDynamicPropertyHandlerClass()</code>.
     */
    public static DynamicPropertyHandler getDynamicPropertyHandler(Class clazz) {
        DynamicPropertyHandler handler = (DynamicPropertyHandler)dynamicPropertyHandlerMap.get(clazz);
        if (handler == null){
            try {
                handler = (DynamicPropertyHandler)clazz.newInstance();
            }
            catch (Exception ex){
                throw new RuntimeException("Cannot allocate dynamic property handler " +
                    " of class " + clazz + ".\n" + ex);
            }
            dynamicPropertyHandlerMap.put(clazz, handler);
        }
        return handler;
    }


}